within TAeZoSysPro.FluidDynamics.BasesClasses;
model MolecularDiffusion

  //Media delcaration
  replaceable package Medium = TAeZoSysPro.Media.MyMedia;

  // User defined parameters
  parameter Modelica.SIunits.Thickness Th = 1 "Thickness between concentration potential difference";
  parameter Modelica.SIunits.CrossSection A = 0;
  parameter Modelica.SIunits.DiffusionCoefficient D=25*1e-6;

  // Internal variables
  Real grad_X[Medium.nX](each quantity="spatial gradient",each unit="m-1") "gradient of mass fraction";
  Modelica.SIunits.MassFlowRate[Medium.nX] m_flow;
  Modelica.SIunits.Density d_mean;
  Modelica.SIunits.Temperature T_mean;
  Modelica.SIunits.SpecificEnthalpy[Medium.nX] h;
  Real J[Medium.nX](each quantity="mass flux rate",each unit="kg.m-2.s-1") "Gradient of mass fraction";
  Modelica.SIunits.MassFraction[Medium.nX] X_a;
  Modelica.SIunits.MassFraction[Medium.nX]      X_b;

  // Imported modules
  TAeZoSysPro.FluidDynamics.Interfaces.FlowPort_a flowPort_a(redeclare package   Medium = Medium) annotation (
    Placement(visible = true, transformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-102, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  TAeZoSysPro.FluidDynamics.Interfaces.FlowPort_b flowPort_b(redeclare package   Medium = Medium) annotation (
    Placement(visible = true, transformation(origin = {100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

equation
  X_a = flowPort_a.d / sum(flowPort_a.d);
  X_b = flowPort_b.d / sum(flowPort_b.d);
  d_mean = 1/2 * (sum(flowPort_a.d) + sum(flowPort_b.d));
  T_mean = 1/2 * (flowPort_a.T + flowPort_b.T);
  h[1] = Medium.enthalpyOfCondensingGas(T_mean);
  h[2] = Medium.enthalpyOfNonCondensingGas(T_mean);

  //

  // for a binary gas mixture
  grad_X = 1 / Th * ( X_b - X_a);
  J[1] = - d_mean * D * grad_X[1];
  J[2] = - J[1];
  m_flow = J * A;


// Port handovers
  flowPort_a.m_flow = m_flow;
  flowPort_a.H_flow = m_flow * h;
  flowPort_a.m_flow = - flowPort_b.m_flow;
  flowPort_a.H_flow + flowPort_b.H_flow = 0.0;

  annotation (
    Documentation(info = "
<html>
  <head>
    <title>MolecularDiffusion</title>
  </head>
        
  <body lang=\"en-UK\">
        
    <p>
      This components allows to model the mass flow rate induced by the gradient of concentration of species within a binary gas mixture. 
      The mass flow rate of species derives from the Fick's equation.
    </p>
                
    <p>
      The physical model under this module uses the following assumptions:
      <ul>
        <li> the total concentration of the mixture is approximately constant </li>
      </ul>
    </p>                
                
    <img        
      src=\"modelica://TAeZoSysPro/Information/FluidDynamics/BasesClasses/EQ_MolecularDiffusion.png\"
    />        
                
    <p>        
      <b>Where</b>:
      <ul>
        <li> <code>m_flow <sub>j</sub> </code> is the mass flow rate of the species 'j' [kg.m<sup>-2</sup>.s<sup>-1</sup>] </li>
        <li> <code>&rho;</code> is the density of the mixture [kg.m<sup>-3</sup>] </li>
        <li> <code>D<sub>ij</sub></code> binary coefficient of diffusion from 'i' to 'j' [m<sup>2</sup>.s<sup>-1</sup>] </li>
        <li> <code>c<sub>j</sub></code> mass fraction of the species 'j' [-] </li>
      </ul>                                
    </p>
                
    <p>                
      The quantities contained in the equation are such that <code>D<sub>ij</sub></code> = <code>D<sub>ji</sub></code> (symmetry of the interaction between particles i and j) and <code>c<sub>i</sub></code> + <code>c<sub>j</sub></code> = 1 (by definition of the mass fraction). 
      It is deduce that diffusion does not globally transport mass, it only distributes it differently:
    </p>

    <img        
      src=\"modelica://TAeZoSysPro/Information/FluidDynamics/BasesClasses/EQ_MolecularDiffusion2.png\"
    />
                    
    <p>                
      Therefore an to avoid mass transport, the global density used in the equations above are computed at the middle of the diffusion length.
    </p>        
                
  </body>
</html>"),
    Icon(graphics={
                     Ellipse(origin = {-52, 80}, fillColor = {0, 85, 255},
            fillPattern =                                                                FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                          Ellipse(origin = {-52, 68}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                     FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, 56}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, 20}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, 44}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, 32}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, -40}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, -28}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, -52}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, -16}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, -4}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, 8}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, -76}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-52, -64}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-40, 62}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-30, 74}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-30, 50}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, 64}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-40, 38}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-30, 26}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-40, 14}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-30, 2}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-40, -10}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-30, -22}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-40, -34}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-30, -46}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-40, -58}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-30, -70}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, 40}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, 16}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, -8}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, -32}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, -56}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, -78}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, 86}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-40, 86}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-40, -80}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Rectangle(fillColor = {0, 170, 255}, extent = {{-56, 88}, {62, -84}}),
                                                                                                                                                                                                                Ellipse(origin = {54, 80}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, 8}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, -4}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, -16}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, 32}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, 44}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, 20}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, 56}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, 68}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, -28}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, -40}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, -52}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, -64}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {54, -76}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {42, 86}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {42, 62}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {42, 38}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {42, 14}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {42, -10}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {42, -34}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {42, -58}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {42, -80}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {30, 74}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {30, 50}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {30, 26}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {30, 2}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {30, -22}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {30, -46}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {30, -70}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, 86}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, 64}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, 40}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, 16}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, -8}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, -32}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, -56}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, -78}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {24, 64}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {24, 42}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {24, 18}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {24, -6}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {24, -30}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {24, -54}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, 76}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, 54}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, 30}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, 6}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, -18}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, -42}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {10, -66}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {24, -76}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {46, 74}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {46, 26}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {46, -22}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {46, -70}, fillColor = {0, 85, 255},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, 76}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, 54}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, 30}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, 6}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, -18}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, -42}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-12, -66}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-24, 64}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-24, 40}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-24, 16}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-24, -8}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-24, -32}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-24, -56}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-24, -80}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-24, 86}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-44, 74}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-44, 26}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-44, -22}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Ellipse(origin = {-44, -70}, fillColor = {255, 0, 0},
            fillPattern =                                                                                                                                                                                                        FillPattern.Solid, extent = {{-2, 2}, {4, -4}}, endAngle = 360),
                                                                                                                                                                                                                Line(origin = {0, 2}, points = {{0, 86}, {0, -86}}, pattern = LinePattern.Dot)}, coordinateSystem(initialScale = 0.1)));
end MolecularDiffusion;
